@@ -69,10 +69,13 @@
  case '8':	\
  case '9'
 
+/* Hexadecimal, in upper and lower case. */
+
 #define UHEX  'A': case 'B': case 'C': case 'D': case 'E': case 'F'
 #define LHEX  'a': case 'b': case 'c': case 'd': case 'e': case 'f'
 
-/* UTF-8 switches. */
+/* As of version 0.45 of JSON::Parse, most of the UTF-8 switches are
+   now in "unicode.c", but the following one is JSON-specific. */
 
 /* This excludes '"' and '\'. */
 
@@ -92,54 +95,6 @@
  case 0x70: case 0x71: case 0x72: case 0x73: case 0x74: case 0x75: case 0x76:\
  case 0x77: case 0x78: case 0x79: case 0x7A: case 0x7B: case 0x7C: case 0x7D:\
  case 0x7E: case 0x7F
-#define BYTE_80_8F \
-      0x80: case 0x81: case 0x82: case 0x83: case 0x84: case 0x85: case 0x86:\
- case 0x87: case 0x88: case 0x89: case 0x8A: case 0x8B: case 0x8C: case 0x8D:\
- case 0x8E: case 0x8F
-#define BYTE_80_9F \
-      0x80: case 0x81: case 0x82: case 0x83: case 0x84: case 0x85: case 0x86:\
- case 0x87: case 0x88: case 0x89: case 0x8A: case 0x8B: case 0x8C: case 0x8D:\
- case 0x8E: case 0x8F: case 0x90: case 0x91: case 0x92: case 0x93: case 0x94:\
- case 0x95: case 0x96: case 0x97: case 0x98: case 0x99: case 0x9A: case 0x9B:\
- case 0x9C: case 0x9D: case 0x9E: case 0x9F
-#define BYTE_80_BF \
-      0x80: case 0x81: case 0x82: case 0x83: case 0x84: case 0x85: case 0x86:\
- case 0x87: case 0x88: case 0x89: case 0x8A: case 0x8B: case 0x8C: case 0x8D:\
- case 0x8E: case 0x8F: case 0x90: case 0x91: case 0x92: case 0x93: case 0x94:\
- case 0x95: case 0x96: case 0x97: case 0x98: case 0x99: case 0x9A: case 0x9B:\
- case 0x9C: case 0x9D: case 0x9E: case 0x9F: case 0xA0: case 0xA1: case 0xA2:\
- case 0xA3: case 0xA4: case 0xA5: case 0xA6: case 0xA7: case 0xA8: case 0xA9:\
- case 0xAA: case 0xAB: case 0xAC: case 0xAD: case 0xAE: case 0xAF: case 0xB0:\
- case 0xB1: case 0xB2: case 0xB3: case 0xB4: case 0xB5: case 0xB6: case 0xB7:\
- case 0xB8: case 0xB9: case 0xBA: case 0xBB: case 0xBC: case 0xBD: case 0xBE:\
- case 0xBF
-#define BYTE_90_BF \
-      0x90: case 0x91: case 0x92: case 0x93: case 0x94: case 0x95: case 0x96:\
- case 0x97: case 0x98: case 0x99: case 0x9A: case 0x9B: case 0x9C: case 0x9D:\
- case 0x9E: case 0x9F: case 0xA0: case 0xA1: case 0xA2: case 0xA3: case 0xA4:\
- case 0xA5: case 0xA6: case 0xA7: case 0xA8: case 0xA9: case 0xAA: case 0xAB:\
- case 0xAC: case 0xAD: case 0xAE: case 0xAF: case 0xB0: case 0xB1: case 0xB2:\
- case 0xB3: case 0xB4: case 0xB5: case 0xB6: case 0xB7: case 0xB8: case 0xB9:\
- case 0xBA: case 0xBB: case 0xBC: case 0xBD: case 0xBE: case 0xBF
-#define BYTE_A0_BF \
-      0xA0: case 0xA1: case 0xA2: case 0xA3: case 0xA4: case 0xA5: case 0xA6:\
- case 0xA7: case 0xA8: case 0xA9: case 0xAA: case 0xAB: case 0xAC: case 0xAD:\
- case 0xAE: case 0xAF: case 0xB0: case 0xB1: case 0xB2: case 0xB3: case 0xB4:\
- case 0xB5: case 0xB6: case 0xB7: case 0xB8: case 0xB9: case 0xBA: case 0xBB:\
- case 0xBC: case 0xBD: case 0xBE: case 0xBF
-#define BYTE_C2_DF \
-      0xC2: case 0xC3: case 0xC4: case 0xC5: case 0xC6: case 0xC7: case 0xC8:\
- case 0xC9: case 0xCA: case 0xCB: case 0xCC: case 0xCD: case 0xCE: case 0xCF:\
- case 0xD0: case 0xD1: case 0xD2: case 0xD3: case 0xD4: case 0xD5: case 0xD6:\
- case 0xD7: case 0xD8: case 0xD9: case 0xDA: case 0xDB: case 0xDC: case 0xDD:\
- case 0xDE: case 0xDF
-#define BYTE_E1_EC \
-      0xE1: case 0xE2: case 0xE3: case 0xE4: case 0xE5: case 0xE6: case 0xE7:\
- case 0xE8: case 0xE9: case 0xEA: case 0xEB: case 0xEC
-#define BYTE_EE_EF \
-      0xEE: case 0xEF
-#define BYTE_F1_F3 \
-      0xF1: case 0xF2: case 0xF3
 
 /* A "string_t" is a pointer into the input, which lives in
    "parser->input". The "string_t" structure is used for copying
@@ -160,7 +115,8 @@ typedef struct string {
 }
 string_t;
 
-typedef enum {
+typedef enum 
+{
     json_invalid,
     json_initial_state,
     json_string,
@@ -187,7 +143,10 @@ const char * type_names[json_overflow] = {
 /* The maximum value of bytes to check for. */
 
 #define JSON3MAXBYTE 0x100
-// uncomment this when running random test to terminal otherwise ...
+
+// uncomment this when running random test to terminal otherwise the
+// random characters will mess up the terminal.
+
 //#define JSON3MAXBYTE 0x80
 
 #include "errors.c"
@@ -230,8 +189,9 @@ typedef struct parser {
 
     /* Where the beginning of the series of unfortunate events
        was. For example if we are parsing an array, this points to the
-       [ at the start of the array, or if we are parsing a string,
-       this points to the byte after " at the start of the string. */
+       "[" at the start of the array, or if we are parsing a string,
+       this points to the byte after the '"' at the start of the
+       string. */
 
     unsigned char * bad_beginning;
 
@@ -381,7 +341,7 @@ static void make_valid_bytes (json_parse_t * parser)
     for (i = 0; i < n_expectations; i++) {
 	int X;
 	X = 1<<i;
-	if (SPECIFIC(X)) {
+	if (SPECIFIC (X)) {
 	    continue;
 	}
 	if (parser->expected & X) {
@@ -391,7 +351,7 @@ static void make_valid_bytes (json_parse_t * parser)
 	    }
 	}
     }
-    if (SPECIFIC(parser->expected)) {
+    if (SPECIFIC (parser->expected)) {
 	parser->valid_bytes[parser->literal_char] = 1;
     }
 }
@@ -622,7 +582,7 @@ failbadinput (json_parse_t * parser)
 	    EROVERFLOW;
 	    joined = 0;
 
-	    if (SPECIFIC(parser->expected)) {
+	    if (SPECIFIC (parser->expected)) {
 		if (! parser->literal_char) {
 		    failbug (__FILE__, __LINE__, parser,
 			     "expected literal character unset");
@@ -633,7 +593,7 @@ failbadinput (json_parse_t * parser)
 	    for (i = 0; i < n_expectations; i++) {
 		int X;
 		X = 1<<i;
-		if (SPECIFIC(X)) {
+		if (SPECIFIC (X)) {
 		    continue;
 		}
 		if (i == xin_literal) {
@@ -1073,12 +1033,14 @@ get_key_string (json_parse_t * parser, string_t * key)
 		else {
 		    goto unitunes;
 		}
+		/* not a fall through, we always "goto" above. */
 	    default:
 		parser->bad_beginning = parser->end - 1 - i;
 		parser->expected = XHEXADECIMAL_CHARACTER;
 		parser->bad_byte = parser->end - 1;
 		UNIFAIL (unexpected_character);
 	    }
+	    /* not a fall through, we either UNIFAIL or goto above. */
 
 	default:
 	    parser->bad_beginning = key->start - 1;
@@ -1086,6 +1048,7 @@ get_key_string (json_parse_t * parser, string_t * key)
 	    parser->bad_byte = parser->end - 1;
 	    STRINGFAIL (unexpected_character);
 	}
+	/* Not a fall through, we never arrive here. */
 
     case BADBYTES:
 
@@ -1093,11 +1056,13 @@ get_key_string (json_parse_t * parser, string_t * key)
 	parser->expected = XSTRINGCHAR;
 	parser->bad_byte = parser->end - 1;
 	STRINGFAIL (unexpected_character);
+	/* Not a fall through, STRINGFAIL does not return. */
 
 #define ADDBYTE 
 #define string_start key_string_next
 #define startofutf8string (key->start)
 #include "utf8-byte-one.c"
+	/* Not a fall through. */
     default:
 
 	parser->bad_beginning = key->start - 1;
@@ -1160,14 +1125,16 @@ get_string (json_parse_t * parser)
 	break;
 
     case '\\':
-	HANDLE_ESCAPES(parser->end, start - 1);
+	HANDLE_ESCAPES (parser->end, start - 1);
 	goto string_start;
 
 #define ADDBYTE (* b++ = c)
 #define startofutf8string start
 #include "utf8-byte-one.c"
 
+	/* Not a fall through. */
     default:
+	/* fall through */
     case BADBYTES:
 	ILLEGALBYTE;
     }
@@ -1350,9 +1317,9 @@ json_token_set_end (json_parse_t * parser, json_token_t * jt, unsigned char * en
 {
     if (jt->end != 0) {
 	int offset = (int) (end - parser->input);
-	fprintf (stderr, "%s:%d: attempt to set end as %d is now %d\n",
-		 __FILE__, __LINE__, offset, jt->end);
-	exit (1);
+	failbug (__FILE__, __LINE__, parser,
+		 "attempt to set end as %d is now %d\n",
+		 offset, jt->end);
     }
 
     switch (jt->type) {
